# [Measurement Generator](@id runmg)
The function uses the [AC Power Flow](@ref acpowerflow) or predefined user data to generate measurements ``\{z_1, \dots, z_n\}`` by corrupting the exact solutions ``\{e_1, \dots, e_n\}`` using the additive white Gaussian noise with variances ``\{\sigma_1^{2}, \dots, \sigma_n^{2}\}``:
```math
 z_i = e_i + \epsilon_i  \\[5pt]
 \epsilon_i \sim \mathcal{N}(0,\,\sigma_i^{2}).
```

The standalone measurement generator produces measurement data in a form suitable for the state estimation function.

Also, to achieve global observability of the power system only with PMUs, we implemented the [optimal placement algorithm](@ref optimalpmu).

---

## Run Settings
The standalone measurement generator receives inputs for measurement variances, and inputs for measurement sets, to produce measurement data. There are two export formats supported for the measurement data, `.h5` or `.xlsx` file. The settings are provided as input arguments of the function `runmg()`.

The order of inputs and their appearance is arbitrary, with only DATA input required. Still, for the methodological reasons, the syntax examples follow a certain order.

#### Syntax
```julia-repl
runmg(DATA)
runmg(DATA; RUNPF)
runmg(DATA; RUNPF, SET)
runmg(DATA; RUNPF, SET, VARIANCE)
runmg(DATA; RUNPF, SET, VARIANCE, ACCONTROL)
runmg(DATA; RUNPF, SET, VARIANCE, ACCONTROL, SAVE)
```
```@raw html
&nbsp;
```
#### Description
```julia-repl
runmg(DATA) computes the AC power flow problem and generates measurements
runmg(DATA; RUNPF) sets the AC power flow analysis
runmg(DATA; RUNPF, SET) defines the measurement set (in-service and out-of-service)
runmg(DATA; RUNPF, SET, VARIANCE) defines measurement values using predefined variances
runmg(DATA; RUNPF, SET, VARIANCE, ACCONTROL) sets variables for the AC power flow
runmg(DATA; RUNPF, SET, VARIANCE, ACCONTROL, SAVE) exports measurements and power system data
```
```@raw html
&nbsp;
```
#### Output
```julia-repl
measurements, system, info = runmg() returns measurement and power system data with a summary
```
```@raw html
&nbsp;
```
#### Examples
```julia-repl
measurements, system, info = runmg("case14.xlsx"; pmuset = "optimal")
```
```julia-repl
measurements, = runmg("case14.xlsx"; pmuset = ["Iij" 5 "Vi" 2])
```
```julia-repl
runmg("case14.xlsx"; pmuset = ["Iij" "all" "Vi" 2], legacyset = ["Pij" 4 "Qi" 8])
```
```julia-repl
runmg("case30.h5"; legacyset = ["redundancy" 3.1], legacyvariance = ["complete" 1e-4])
```
```julia-repl
runmg("case30.h5"; legacyset = "complete", legacyvariance = ["Pij" 1e-4])
```
---

## Variable Arguments
The measurement generator function `runmg()` receives the variable number of argument DATA.

| DATA            | input power system with/without measurements data                                         |
|:----------------|:------------------------------------------------------------------------------------------|
|                 |                                                                                           |
| **Example**     | `"case14.h5"`                                                                             |
| **Description** |  loads the power system with/without measurements data using h5-file from the package     |
|                 |                                                                                           |
| **Example**     | `"case14.xlsx"`                                                                           |
| **Description** |  loads the power system with/without measurements data using xlsx-file from the package   |
|                 |                                                                                           |
| **Example**     | `"C:/case14.h5"`                                                                          |
| **Description** |  loads the power system with/without measurements data using h5-file from a custom path   |
|                 |                                                                                           |
| **Example**     | `"C:/case14.xlsx"`                                                                        |                                     
| **Description** |  loads the power system with/without measurements data using xlsx-file from a custom path |

---

## Keyword Arguments
The measurement generator function `runmg()` receives the group of arguments by keyword: RUNPF, SET, VARIANCE, ACCONTROL and SAVE.

| RUNPF           | sets the AC power flow analysis                                                        |
|:----------------|:---------------------------------------------------------------------------------------|
|                 |                                                                                        |
| **Command**     | `runflow = 1`                                                                          |
| **Description** |  forces the AC power flow analysis to generate measurements, `default RUNPF setting`   |
|                 |                                                                                        |
| **Command**     | `runflow = 0`                                                                          |
| **Description** |  generates measurements directly from the input DATA                                   |

```@raw html
&nbsp;
```

| SET             | defines the measurement set (in-service and out-of-service)                            |
|:----------------|:---------------------------------------------------------------------------------------|
|                 |                                                                                        |
| **Command**     | `pmuset = "complete"`                                                                  |
| **Description** |  complete phasor measurement set in-service                                            |
|                 |                                                                                        |
| **Command**     | `pmuset = "optimal"`                                                                   |
| **Description** |  deploys phasor measurements according to the optimal PMU location using GLPK solver, where the system is completely observable only by phasor measurements |
|                 |                                                                                        |
| **Command**     | `pmuset = ["redundancy" value]`                                                        |
| **Description** |  deploys random angle and magnitude measurements measured by PMUs according to the corresponding redundancy |
|                 |                                                                                        |
| **Command**     | `pmuset = ["device" value]`                                                            |
| **Description** | deploys voltage and current phasor measurements according to the random selection of PMUs placed on buses, to deploy all devices use `"all"` as value |
|                 |                                                                                        |
| **Command**     | `pmuset = ["Iij" value "Dij" value "Vi" value "Ti" value "complete"]`                  |
| **Description** | deploys phasor measurements according to the random selection of measurement types[^1], to deploy all selected measurements use `"all"` as `value`, to deploys all measurements in-service, except for those individually defined use the keyword `"complete"` |
|                 |                                                                                        |
| **Command**     | `legacyset = "complete"`                                                               |
| **Description** |  complete legacy measurement set in-service                                            |
|                 |                                                                                        |
| **Command**     | `legacyset = ["redundancy " value]`                                                    |
| **Description** |  deploys random selection of legacy measurements according the corresponding redundancy |
|                 |                                                                                        |
| **Command**     | `legacyset = ["Pij" value "Qij" value "Iij" value "Pi" value "Qi" value "Vi" value "complete"]` |
| **Description** |  deploys legacy measurements according to the random selection of measurement types[^2], to deploy all selected measurements use `"all"` as `value`, to deploys all measurements in-service, except for those individually defined use the keyword `"complete"` |


```@raw html
&nbsp;
```

!!! note "SET"
    If `runflow = 0`, the function keeps measurement set as in the input DATA and changes only the sets that are called using keywords. For example, if the keywords `pmuset` and `legacyset` are omitted, the function will retain the measurement set as in the input DATA, which allows the same measurement set, while changing the measurement variances.

    If `runflow = 1`, the function starts with all the measurement sets marked as out-of-service.  

    Further, the function accept any subset of phasor[^1] or legacy[^2] measurements, and consequently, it is not necessary to define attributes for all measurements.  
    ```julia-repl
    runmg("case14.h5"; runflow = 1, pmuset = ["Iij" "all" "Vi" 2])
    ```
    Thus, the measurement set will consist of two randomly selected bus voltage magnitude measurements, and all branch current magnitude measurements, both of them related with PMUs.  

```@raw html
&nbsp;
```

| VARIANCE        | defines measurement values using predefined variances                                                       |
|:----------------|:------------------------------------------------------------------------------------------------------------|
|                 |                                                                                                             |
| **Command**     | `pmuvariance = ["complete" value]`                                                                          |
| **Description** | applies fixed-value variance over all phasor measurements                                                   |
|                 |                                                                                                             |
| **Command**     | `pmuvariance = ["random" min max]`                                                                          |
| **Description** | selects variances uniformly at random within limits, applied over all phasor measurements                   |
|                 |                                                                                                             |
| **Command**     | `pmuvariance = ["Iij" value "Dij" value "Vi" value "Ti" value "complete" value]`                            |
| **Description** | predefines variances over a given subset of phasor measurements[^1]; to apply fixed-value variance over all, except for those individually defined use the name-value pair setting `"complete" value` |
|                 |                                                                                                             |
| **Command**     | `legacyvariance = ["complete" value]`                                                                       |
| **Description** | applies fixed-value variance over all legacy measurements                                                   |
|                 |                                                                                                             |
| **Command**     | `legacyvariance = ["random" min max]`                                                                       |
| **Description** |  selects variances uniformly at random within limits, applied over all legacy measurements                  |
|                 |                                                                                                             |
| **Command**     | `legacyvariance = ["Pij" value "Qij" value "Iij" value "Pi" value "Qi" value "Vi" value "complete" value]`  |
| **Description** |  predefines variances over a given subset of legacy measurements[^2], to apply fixed-value variance over all, except for those individually defined use the name-value pair setting `"complete" value` |

```@raw html
&nbsp;
```

!!! note "VARIANCE"
    If `runflow = 0`, the function keeps measurement values and measurement variances as in the input DATA, and changes only measurement values and variances that are called using keywords. For example, if the keywords `pmuvariance` and `legacyvariance` are omitted, the function will retain the measurement values and variances as in the input DATA, allowing the same measurement values and variances, while changing the measurement sets.

    If `runflow = 1`, the function starts with zero variances, meaning that measurement values are equal to zero values.

    Further, the function accepts any subset of phasor[^1] or legacy[^2] measurements, consequently, it is not necessary to define attributes for all measurements, where keyword `"complete"` generates measurement values according to defined variance for all measurements, except for those individually defined.   
    ```julia-repl
    runmg("case14.h5"; runflow = 1, legacyvariance = ["Pij" 1e-4 "complete" 1e-5])
    ```
    The function applies variance value of 1e-5 over all legacy measurements, except for active power flow measurements which have variance equal to 1e-4.

```@raw html
&nbsp;
```
| ACCONTROL       | sets variables for the AC power flow                                                           |
|:----------------|:-----------------------------------------------------------------------------------------------|
|                 |                                                                                                |
| **Command**     | `max = value`                                                                                  |
| **Description** | specifies the maximum number of iterations for the AC power flow, `default setting: max = 100` |
|                 |                                                                                                |
| **Command**     | `stop = value`                                                                                 |
| **Description** | specifies the stopping criteria for the AC power flow, `default setting: stop = 1.0e-8`        |
|                 |                                                                                                |
| **Command**     | `reactive = 1`                                                                                 |
| **Description** | forces reactive power constraints for the AC power flow, `default setting: reactive = 0`       |
|                 |                                                                                                |
| **Command**     | `solve = "builtin"`                                                                            |
| **Description** |  built-in linear system solver, `default solve setting`                                        |
|                 |                                                                                                |
| **Command**     | `solve = "lu"`                                                                                 |
| **Description** |  LU linear system solver                                                                       |

```@raw html
&nbsp;
```
| SAVE            | exports results                  |
|:----------------|:---------------------------------|
|                 |                                  |
| **Command**     | `save = "path/name.h5"`          |
| **Description** |  saves results in the h5-file    |
|                 |                                  |
| **Command**     | `save = "path/name.xlsx"`        |
| **Description** |  saves results in the xlsx-file  |

---

## Data Structure
The function supports two input types: `.h5` or `.xlsx` file extensions. The measurement data structure consists of variables `pmuVoltage` and `pmuCurrent` associated with phasor measurements, and `legacyFlow`, `legacyCurrent`, `legacyInjection` and `legacyVoltage` associated with legacy measurements. Further, the function requires knowledge about a power system using variables `bus`, `branch`, `generator` and `basePower`.

The minimum amount of information within an instance of the data structure required to run the module requires one variable associated with measurements if `runflow = 0`, and `bus` and `branch` variables.

We advise the reader to read the sections [Power System Data Structure](@ref powersysteminputdata) and [Measurement Data Structure](@ref measurementinputdata) which provides the structure of the DATA.

---

## Flowchart

The measurement generator flowchart depicts the algorithm process according to user settings.

![](../assets/generatorm_chart.svg)


[^1]: Complete phasor measurement set contains branch current magnitude Iij, branch current angle Dij, bus voltage magnitude Vi and bus voltage angle Ti measurements.
[^2]: Complete legacy measurement set contains active power flow Pij, reactive power flow Qij, branch current magnitude Iij, active power injection Pi, reactive power injection Qi and bus voltage magnitude Vi measurements.
